package agileTraveler.learn;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertTrue;
import static junit.framework.Assert.fail;
import static org.junit.Assert.*;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;

import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.TagNode;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

public class WeatherXMLParser {
	private static InputSource google;

	@BeforeClass
	public static void downloadXml() throws UnsupportedEncodingException {
		google = getXml("São Paulo");
	}

	private static InputSource getXml(String city)
			throws UnsupportedEncodingException {
		String url = "http://www.google.com/ig/api?weather=";
		InputSource xml = null;
		url += URLEncoder.encode(city, "UTF-8");
		xml = new InputSource(url);
		xml.setEncoding("ISO-8859-1");
		return xml;
	}

	@Test
	public void shouldReadCurrentConditionWithJDOM() throws JDOMException,
			IOException {
		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(google);
		Element root = doc.getRootElement();
		String currentCondition = root.getChild("weather").getChild(
				"current_conditions").getChild("condition")
				.getAttribute("data").getValue();
		assertTrue(currentCondition.length() > 0);
	}
	
	@Test
	public void shouldReadCurrentConditionXerces2SAX() {
		SAXBuilder builder = new SAXBuilder();
		try {
			Document doc = builder.build(google);
			Element root = doc.getRootElement();
			String currentCondition = root.getChild("weather").getChild(
					"current_conditions").getChild("condition").getAttribute(
					"data").getValue();
			assertTrue(currentCondition.length() > 0);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}

	@Test
	public void shouldParseHtml() throws Exception {
		// "http://www.wunderground.com/history/airport/SBSP/2009/3/23/MonthlyHistory.html";
		String url = "http://127.0.0.1/MonthlyHistory.html";
		HtmlCleaner cleaner = new HtmlCleaner();
		TagNode node = cleaner.clean(new URL(url));
		TagNode[] tagNodes = node.getElementsByName("span", true);
		String temp = tagNodes[12].getText().toString();
		assertEquals(31, Integer.parseInt(temp));
	}

	@Test
	// SAX is recommended over DOM public void
	public void shouldReadCurrentConditionXerces2DOM() {
		DOMParser parser = new DOMParser();
		try {
			parser.parse(google);
			assertNotNull(parser);
			org.w3c.dom.Document doc = parser.getDocument();
			NodeList list = doc.getElementsByTagName("condition");
			String currentCondition = list.item(0).getAttributes().item(0)
					.getNodeValue();
			assertTrue(currentCondition.length() > 0);
		} catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}

	/*
	@Test
	public void shouldParseHtmlWithJsoup() throws MalformedURLException,
			IOException {
		String url = "http://127.0.0.1/MonthlyHistory.html";
		org.jsoup.nodes.Document doc = Jsoup.parse(new URL(url), 3000);
		assertNotNull(doc);
		int temp = Integer.parseInt(doc.getElementsByTag("table").get(9)
				.getElementsByClass("b").get(1).text());
		assertEquals(31, temp);
	}

	@Ignore
	public void shouldParseHtmlWithTagSoup() throws Exception {
		Parser parser = new Parser();
		SAX2DOM sax2dom = new SAX2DOM();
		parser.setContentHandler(sax2dom);
		parser.setFeature(Parser.namespacesFeature, false);
		String url = "http://127.0.0.1/MonthlyHistory.html";
		InputSource html = new InputSource(url);
		parser.parse(html);
		Node root = sax2dom.getDOM();
		assertEquals("bla", root.toString());
	}

	@Ignore
	public void shouldParseHtmlNeko() throws Exception {
		String url = "http://www.w3.org/";
		InputSource weather = new InputSource(url);
		org.cyberneko.html.parsers.DOMParser parser = new org.cyberneko.html.parsers.DOMParser();
		parser.parse(weather);
		Node root = parser.getDocument();
		assertEquals("bla", root.getFirstChild().toString());
	}
	*/
}
