package agileTraveler.airTicketPrice;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.TagNode;

import agileTraveler.exceptions.AirTicketPricesNotFoundException;
import agileTraveler.exceptions.CityNotFoundException;

public class AirCompanyTicketPricesParser implements
		IAirCompanyTicketPricesParser {

	private String url;

	public AirCompanyTicketPricesParser(String departureAirport,
			String returnAirport, Date departureDate, Date returnDate) {
		SimpleDateFormat df = new SimpleDateFormat("MM/dd/yyyy");
		String dd = df.format(departureDate);
		String rd = df.format(returnDate);
		createURL(departureAirport, returnAirport, dd, rd);
	}

	public AirCompanyTicketPricesParser(String departureAirport,
			String returnAirport, String departureDate, String returnDate) {
		createURL(departureAirport, returnAirport, departureDate, returnDate);
	}

	public void setURL(String localURL) {
		url = localURL;
	}

	private void createURL(String departureAirport, String returnAirport,
			String departureDate, String returnDate) {
		url = "http://www.expedia.com/pub/agent.dll?"
				+ "qscr=fexp&flag=q&city1="
				+ departureAirport
				+ "&citd1="
				+ returnAirport
				+ "&date1="
				+ departureDate
				+ "&time1=362&date2="
				+ returnDate
				+ "&time2=362&cAdu=1&cSen=&cChi=&cInf=&infs=2&tktt=3&trpt=2&ecrc=&eccn=&qryt=8"
				+ "&rdct=1&mdpcid=META.kayak.Comp.FLT.front_door_unchecked.2010campaign";
	}

	@Override
	public List<AirCompanyTicketPrices> getAirTicketPrices()
			throws MalformedURLException, IOException, CityNotFoundException,
			AirTicketPricesNotFoundException {
		TagNode node = prepareHTMLParser();
		List<AirCompanyTicketPrices> ticketPricesCompanies = parserAllAirTicketCompanies(node);
		System.out.println("INFO: finished air tickets");
		
		return ticketPricesCompanies;
	}

	private List<AirCompanyTicketPrices> parserAllAirTicketCompanies(
			TagNode node) throws CityNotFoundException,
			AirTicketPricesNotFoundException {
		List<AirCompanyTicketPrices> ticketPricesCompanies = new ArrayList<AirCompanyTicketPrices>();

		verifyPage(node);
		int numberOfCompanies = getNumberOfCompanies(node);

		for (int i = 0; i < numberOfCompanies; i++)
			ticketPricesCompanies.add(parseTicketPriceCompany(node, i));
		verifyIfNoPricesForOneOrNonStop(ticketPricesCompanies);
		return ticketPricesCompanies;
	}

	private TagNode prepareHTMLParser() throws IOException,
			MalformedURLException {
		HtmlCleaner cleaner = new HtmlCleaner();
		System.out.println("INFO: downloading url " + url);
		TagNode node = cleaner.clean(new URL(url));
		return node;
	}

	public int getNumberOfCompanies(TagNode node) {
		int numberOfCompanies = node.getElementsByAttValue("class", "ghead",
				true, false).length;
		return (numberOfCompanies > 5) ? 5 : numberOfCompanies;
	}

	private AirCompanyTicketPrices parseTicketPriceCompany(TagNode node, int i) {
		AirCompanyTicketPrices ticketPricesCompany = new AirCompanyTicketPrices();

		String logoURL = node.getElementsByAttValue("class", "ghead", true,
				false)[i].getElementsByName("IMG", true)[0]
				.getAttributeByName("src");

		ticketPricesCompany.setLogoURL(logoURL);

		String name = node.getElementsByAttValue("class", "gtext", true, false)[1 + i * 2]
				.getText().toString();
		ticketPricesCompany.setName(name.trim());

		TagNode firstCell = node.getElementsByAttValue("id", "ele_1_" + i
				+ "_0", true, false)[0];
		ticketPricesCompany.setPriceForNonStop(getPrice(firstCell));

		TagNode secondCell = node.getElementsByAttValue("id", "ele_1_" + i
				+ "_1", true, false)[0];
		ticketPricesCompany.setPriceForOneStop(getPrice(secondCell));
		return ticketPricesCompany;
	}

	private TagNode getPrevious(TagNode node) {
		TagNode parent = node.getParent();
		int index = parent.getChildren().indexOf(node);
		return (TagNode) parent.getChildren().get(index - 1);
	}

	private String getPrice(TagNode cell) {
		TagNode scriptNode = getPrevious(cell);
		String text = scriptNode.getText().toString().trim();
		if (text.matches(".*<B>.*") && !text.matches(".*tyP.*")) {
			text = text.replaceAll(".*<B>", "");
			text = text.replaceAll("</B>.*", "");
			text = "from " + text;
		} else
			text = "---";
		return text;
	}

	private void verifyPage(TagNode node) throws CityNotFoundException,
			AirTicketPricesNotFoundException {
		verifyIfAirportWasNotFound(node);
		verifyIfThereIsntAnyAirTickets(node);
	}

	private void verifyIfThereIsntAnyAirTickets(TagNode node)
			throws AirTicketPricesNotFoundException {

		String ticketsNotFoundString = "No flights were found that matched your search";
		if (verifyIfTitleMatchesWith(ticketsNotFoundString, node))
			throw new AirTicketPricesNotFoundException();
	}

	private void verifyIfAirportWasNotFound(TagNode node)
			throws CityNotFoundException {
		TagNode[] errors = node.getElementsByAttValue("class", "errorTextStd",
				true, false);
		if (errors.length > 0)
			throw new CityNotFoundException("Could not find any airport");
	}

	private boolean verifyIfTitleMatchesWith(String string, TagNode node) {
		CharSequence StringInCharSequence = string.subSequence(0, string
				.length());
		String pageTitle = node.getElementsByName("title", true)[0].getText()
				.toString();
		return (pageTitle.contains(StringInCharSequence));
	}
	
	private void verifyIfNoPricesForOneOrNonStop(
			List<AirCompanyTicketPrices> ticketPricesCompanies) throws AirTicketPricesNotFoundException {
		
		for (AirCompanyTicketPrices airCompanyTicketPrices : ticketPricesCompanies) {
			if(verifyIfThisCompanyHaveFlightForOneOrNonStop(airCompanyTicketPrices))
				return;
		}
		
		throw new AirTicketPricesNotFoundException();		
	}
	
	private boolean verifyIfThisCompanyHaveFlightForOneOrNonStop(
			AirCompanyTicketPrices airCompanyTicketPrices) {
		
		return (!airCompanyTicketPrices.getPriceForNonStop().equals("---") ||
				!airCompanyTicketPrices.getPriceForOneStop().equals("---"));
	}
}
