package agileTraveler.airTicketPrice;

import java.util.Date;
import java.util.List;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.utilities.DateUtils;

public class AirTicketSearcher {

	public List<AirCompanyTicketPrices> search(String dpAirportCode,
			String dpAirportName, String dtAirportCode, String dtAirportName,
			Date departureDate, Date returnDate, String mockIniSection)
			throws AirTicketSearchException {

		checkFields(dpAirportCode, dtAirportCode, departureDate, returnDate);

		IAirCompanyTicketPricesParser parser;
		if (mockIniSection == null) {
			parser = new AirCompanyTicketPricesParser(dpAirportCode,
					dtAirportCode, departureDate, returnDate);
		} else {
			System.out.println("INFO: parsing .ini for Air Ticket Prices.");
			try {
				parser = new AirCompanyTicketPricesParserMock(mockIniSection);
			} catch (CityNotFoundException e) {
				throw new AirTicketSearchException(e.getMessage(), e);
			} catch (Exception e) {
				throw new AirTicketSearchException(e);
			}
		}

		try {
			return parser.getAirTicketPrices();
		} catch (Exception e) {
			throw new AirTicketSearchException("No air tickets found from "
					+ dpAirportName + " to " + dtAirportName + ".", e);
		}
	}

	private void checkFields(String dpAirportCode, String dtAirportCode,
			Date departureDate, Date returnDate)
			throws AirTicketSearchException {
		if (!DateUtils.validateAll(departureDate, returnDate)) {
			throw new AirTicketSearchException(
					"Couldn't search for air tickets. Please, check the date fields.");
		}
		if (dpAirportCode.length() == 0 || dtAirportCode.length() == 0) {
			throw new AirTicketSearchException(
					"Couldn't search for air tickets because of missing airport in departure or destination city.");
		}
	}

}
