package agileTraveler.bean;

import static agileTraveler.utilities.DateUtils.isFutureDate;
import static agileTraveler.utilities.DateUtils.isPeriodValid;
import static agileTraveler.utilities.DateUtils.stringToDate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import agileTraveler.database.Database;
import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.City;
import agileTraveler.model.Suggestion;
import agileTraveler.utilities.ThreadUtils;

public class Search {

	private String departureAirportName;
	private String departureAirportCode;

	private String destinationAirportName;
	private String destinationAirportCode;
	private String destinationLocationCode;

	private String departureDate;
	private String returnDate;

	private String errorMessage;
	private String airTicketErrorMessage;

	private WeatherBean weatherBean;
	private TravelGuideBean travelGuideBean;
	private String cityName;
	private AirTicketBean ticketPriceBean = null;

	public Search() {
		weatherBean = new WeatherBean();
		travelGuideBean = new TravelGuideBean();
		errorMessage = "";
	}

	public List<Suggestion> autocomplete(Object namePart) throws SQLException,
			ClassNotFoundException {
		if ("null".equals("" + namePart))
			return new ArrayList<Suggestion>();
		List<City> cities = Database.getDatabase().getCities(
				namePart.toString());
		return Suggestion.getSuggestions(cities);
	}

	public void setDestinationAirportName(String destinationCity) {
		this.destinationAirportName = destinationCity;
	}

	public String getDestinationAirportName() {
		return destinationAirportName;
	}

	public void setDepartureAirportName(String departureCity) {
		this.departureAirportName = departureCity;
	}

	public String getDepartureAirportName() {
		return departureAirportName;
	}

	public String search() throws SQLException, ClassNotFoundException,
			CityNotFoundException {

		errorMessage = "";
		cityName = searchCityName();

		Date departure = stringToDate(departureDate);
		Date _return = stringToDate(returnDate);
		dateErrorMessages(departure, _return);

		doSearches(departure, _return);

		return "";
	}

	private void doSearches(Date departure, Date _return) {
		Thread[] ths = new Thread[3];
		try {
			ths[0] = searchTravelGuides(departure, _return);
			ths[1] = searchWeather(departure, _return);
			ths[2] = searchAirTicket(departure, _return);
			ThreadUtils.joinAll(ths);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private Runnable getRunnable(final String method, final Date departure,
			final Date _return) {
		return new Runnable() {
			public void run() {
				try {
					Search.this.getClass().getMethod(method, Date.class,
							Date.class).invoke(Search.this, departure, _return);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		};
	}

	private Thread searchAirTicket(final Date departure, final Date _return) {
		Runnable run = getRunnable("doAirTicketSearch", departure, _return);
		return ThreadUtils.runInOtherThread(run, "Search air ticket thread");
	}

	public void doAirTicketSearch(final Date departure, final Date _return) {
		ticketPriceBean = new AirTicketBean(departureAirportName,
				destinationAirportName, departureAirportCode,
				destinationAirportCode, departure, _return);
	}

	private Thread searchWeather(Date departure, Date _return) {
		Runnable run = getRunnable("doSearchWeather", departure, _return);
		return ThreadUtils.runInOtherThread(run, "Search weather thread");
	}

	public void doSearchWeather(Date departure, Date _return) {
		weatherBean.searchWeather(cityName, departure, _return);
	}

	private void dateErrorMessages(Date departure, Date _return) {
		if (departure == null)
			errorMessage += "Invalid date of departure. ";
		else if (!isFutureDate(departure))
			errorMessage += "Date of departure should not be in the past. ";
		if (_return == null)
			errorMessage += "Invalid date of return. ";
		if (departure != null && _return != null
				&& !isPeriodValid(departure, _return))
			errorMessage += "Date of return should be after date of departure. ";
	}

	private Thread searchTravelGuides(Date departure, Date _return) {
		Runnable run = getRunnable("doSearchTravelGuides", departure, _return);
		return ThreadUtils.runInOtherThread(run, "Search travel guides thread");
	}

	public void doSearchTravelGuides(Date departure, Date _return) {
		travelGuideBean.searchTravelGuide(cityName);
	}

	private String searchCityName() throws SQLException, CityNotFoundException,
			ClassNotFoundException {
		if (destinationLocationCode.length() > 0)
			return Database.getDatabase().getCityName(destinationLocationCode);
		return destinationAirportName;
	}

	public void setDepartureDate(String departureDate) {
		this.departureDate = departureDate;
	}

	public void setReturnDate(String returnDate) {
		this.returnDate = returnDate;
	}

	public String getReturnDate() {
		return returnDate;
	}

	public String getDepartureDate() {
		return this.departureDate;
	}

	public WeatherBean getWeatherBean() {
		return this.weatherBean;
	}

	public TravelGuideBean getTravelGuideBean() {
		return this.travelGuideBean;
	}

	public AirTicketBean getAirTicketBean() {
		return this.ticketPriceBean;
	}

	public void setErrorMessage(String errorMessage) {
		this.errorMessage += errorMessage + " ";
	}

	public String getErrorMessage() {
		return errorMessage;
	}

	public void setDestinationAirportCode(String destinationAirportCode) {
		this.destinationAirportCode = destinationAirportCode;
	}

	public String getDestinationAirportCode() {
		return destinationAirportCode;
	}

	public void setDestinationLocationCode(String destinationLocationCode) {
		this.destinationLocationCode = destinationLocationCode;
	}

	public String getDestinationLocationCode() {
		return destinationLocationCode;
	}

	public void setDepartureAirportCode(String departureAirportCode) {
		this.departureAirportCode = departureAirportCode;
	}

	public String getDepartureAirportCode() {
		return departureAirportCode;
	}

	public String getCityName() {
		return cityName;
	}

	public String getAirTicketErrorMessage() {
		return airTicketErrorMessage;
	}
}
