package agileTraveler.bean;

import java.util.Date;
import java.util.List;

import agileTraveler.model.Weather;
import agileTraveler.utilities.DateUtils;
import agileTraveler.utilities.ThreadUtils;
import agileTraveler.weather.WeatherForecast;
import agileTraveler.weather.WeatherHistory;
import agileTraveler.weather.WeatherSearchException;
import agileTraveler.weather.WeatherSearcher;

public class WeatherBean {

	private Weather weather;
	private List<WeatherForecast> weatherForecasts;
	private List<WeatherHistory> weatherHistories;
	private String errorMessages;
	private String destinationCity;
	private Date departureDate;
	private Date returnDate;
	private WeatherSearcher searcher;
	private static String weatherFactoryClassName;

	public void searchWeather(String destinationCity, Date departureDate, Date returnDate) {
		this.destinationCity = destinationCity;
		this.departureDate = departureDate;
		this.returnDate = returnDate;
		errorMessages = null;
		try {
			searcher = new WeatherSearcher(weatherFactoryClassName);
			doSearch();
		} catch (WeatherSearchException e) {
			setErrorMessages("No weather information found for "
					+ destinationCity  + "(" + e.getMessage() + ")");
		}
	}

	private void doSearch() {
		try {
			Thread ths[] = new Thread[2];
			ths[0] = currentAndForecastThread();
			ths[1] = historyThread();
			ThreadUtils.joinAll(ths);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private Thread currentAndForecastThread() {
		Runnable run = getRunnable("searchCurrentAndForecast");
		return ThreadUtils.runInOtherThread(run, "Weather current Thread");
	}

	private Thread historyThread() {
		Runnable run = getRunnable("searchHistory");
		return ThreadUtils.runInOtherThread(run, "Weather history Thread");
	}

	public void searchCurrentAndForecast() {
		System.out.println("Starting current weather search for: " + destinationCity);
		try {
			weather = searcher.getCurrentWeather(destinationCity);
			weatherForecasts = searcher.getWeatherForecast(destinationCity);
		} catch (WeatherSearchException e) {
			setErrorMessages("Weather information issue: " + e.getMessage());
		}

	}
	
	public void searchHistory() {
		System.out.println("Starting history weather search for: " + destinationCity);
		if (DateUtils.validateAll(departureDate, returnDate)) {
			weatherHistories = searcher.getWeatherHistory(destinationCity, departureDate, returnDate);
		} else {
			weatherHistories = null;
		}
	}
	
	
	private Runnable getRunnable(final String method) {
		return new Runnable() {
			public void run() {
				try {
					WeatherBean.this.getClass().getMethod(method).invoke(
							WeatherBean.this);
				} catch (Exception e) {
					setErrorMessages(e.getMessage());
				}
			}
		};		
	}

	public Weather getWeather() {
		return weather;
	}

	public boolean getRenderWeather() {
		return weather != null;
	}

	public boolean getRenderWeatherHistory() {
		return weatherHistories != null;
	}

	public List<WeatherForecast> getWeatherForecasts() {
		return weatherForecasts;
	}

	public List<WeatherHistory> getWeatherHistories() {
		return weatherHistories;
	}

	public void setErrorMessages(String errorMessages) {
		if (this.errorMessages == null) {
			this.errorMessages = errorMessages;
		} else {
			this.errorMessages += errorMessages;
		}
		this.errorMessages += ". ";
	}

	public String getErrorMessages() {
		return errorMessages;
	}
	
	public static void setSearchWeatherEngine (String searchEngineClass) {
		weatherFactoryClassName = searchEngineClass;
	}
}
