package agileTraveler.travelGuide;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.InputSource;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.TravelGuide;

public class TravelGuideParser implements ITravelGuideParser {

	private static final String END_POINT = "ecs.amazonaws.com";
	private static final String AWS_ACCESS_KEY_ID = "AKIAISF25AI63ZVZOKIA";
	private static final String AWS_SECRET_KEY = "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";

	private Element xmlItems;

	private String city;
	private int page;
	private int numberOfPages;
	private List<TravelGuide> travelGuides;
	private InputStream mockInputStream = null;

	public void setCityCode(String city) throws JDOMException, IOException,
			CityNotFoundException {
		this.city = city;
		this.page = 1;
		travelGuides = parseTravelGuides();
	}
	
	public void firstPage() throws JDOMException, IOException, CityNotFoundException{
		page = 1;
		travelGuides = parseTravelGuides();
	}
	
	public void previousPage() throws JDOMException, IOException, CityNotFoundException{
		if (page == 1)
			return;
		page--;
		travelGuides = parseTravelGuides();
	}
	
	public void nextPage() throws JDOMException, IOException, CityNotFoundException {
		if (page == numberOfPages)
			return;
		page++;
		travelGuides = parseTravelGuides();
	}

	public void lastPage() throws JDOMException, IOException, CityNotFoundException {
		page = numberOfPages;
		travelGuides = parseTravelGuides();
	}
	
	public Collection<TravelGuide> getTravelGuides() {
		return travelGuides;
	}

	private Element getXmlItems() throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		Document doc = getDocument(builder);
		Element items = (Element) doc.getRootElement().getChildren().get(1);
		return items;
	}

	private Document getDocument(SAXBuilder builder) throws JDOMException,
			IOException {
		Document doc = null;
		
		if (mockInputStream == null) {
			String url = getURL();
			System.out.println("INFO: downloading url " + url);
			doc = builder.build(new InputSource(url));
		} else {
			System.out.println("INFO: parsing local file for Travel Guides.");
			doc = builder.build(mockInputStream);
		}
		
		return doc;
	}

	private String getURL() {
		SignedRequestsHelper signature = new SignedRequestsHelper(END_POINT,
				AWS_ACCESS_KEY_ID, AWS_SECRET_KEY);

		Map<String, String> params = new HashMap<String, String>();
		params.put("Service", "AWSECommerceService");
		params.put("Operation", "ItemSearch");
		params.put("ItemPage", "" + page);
		params.put("SearchIndex", "Books");
		params.put("Power", "subject:" + city + " Travel Guide");

		String url = signature.sign(params);
		return url;
	}

	@SuppressWarnings("unchecked")
	private ArrayList<TravelGuide> parseTravelGuides()
			throws CityNotFoundException, JDOMException, IOException {
		xmlItems = getXmlItems();
		
		ArrayList<TravelGuide> travelGuides = new ArrayList<TravelGuide>();
		List<Element> items = xmlItems.getChildren();

		int results = Integer.parseInt(items.get(1).getText());
		numberOfPages = Integer.parseInt(items.get(2).getText());

		if (results == 0)
			throw new CityNotFoundException(city);

		for (int i = 3; i < items.size(); i++) {
			Element item = items.get(i);
			List<Element> travelGuideXml = ((Element) item.getChildren().get(2))
					.getChildren();
			String title = travelGuideXml.get(travelGuideXml.size() - 1)
					.getText();
			String url = ((Element) item.getChildren().get(1)).getText();
			TravelGuide travelGuide = new TravelGuide(title, url);
			travelGuides.add(travelGuide);
		}
		
		System.out.println("INFO: finished travel guides");
		
		return travelGuides;
	}

	public int getPage(){
		return page;
	}
	
	public int getNumberOfPages() {
		return numberOfPages;
	}

	public void setInputStream(InputStream mockInputStream) {
		this.mockInputStream  = mockInputStream; 
	}

}
