package agileTraveler.travelGuide;

import java.util.Collection;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.TravelGuide;

public class TravelGuideSearcher {

	public Collection<TravelGuide> searchTravelGuides (String destinationCity, String mockIniSection) throws TravelGuideSearchException {
		return (mockIniSection == null) ? 
				search(destinationCity) :
			    searchMock(destinationCity, mockIniSection);
	}
	
	private Collection<TravelGuide> search (String destinationCity) throws TravelGuideSearchException {
		ITravelGuideParser parser = new TravelGuideParser();
		return getTravelGuides(destinationCity, parser);		
	}
	
	private Collection<TravelGuide> searchMock (String destinationCity, String mockIniSection) throws TravelGuideSearchException {
			ITravelGuideParser parser = null;
			try {
				parser = new TravelGuideParserMock(mockIniSection);
			} catch (CityNotFoundException e) {
				exceptionForCityNotFound(destinationCity, e);
			} catch (Exception e) {
				throw new TravelGuideSearchException(e.getMessage(), e);
			}
			return getTravelGuides(destinationCity, parser);	
	}
	
	private Collection<TravelGuide> getTravelGuides(String destinationCity, ITravelGuideParser parser) throws TravelGuideSearchException {
		try {
			parser.setCityCode(destinationCity);
			return parser.getTravelGuides();
		} catch (CityNotFoundException e) {
			exceptionForCityNotFound(destinationCity, e);
		} catch (Exception e) {
			throw new TravelGuideSearchException(e.getMessage(), e);
		}
		return null;//never will get here
	}

	private void exceptionForCityNotFound(String destinationCity,
			CityNotFoundException e) throws TravelGuideSearchException {
		throw new TravelGuideSearchException(
				"We did not find any travel guide for "	+ destinationCity, e);
	}
	
}
