package agileTraveler.utilities;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;

public class DateUtils {

	public static boolean isPeriodValid(Date dateBegin, Date dateEnd) {
		return dateBegin.compareTo(dateEnd) <= 0;
	}

	public static boolean isFutureDate(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return isPeriodValid(calendar.getTime(), date);
	}

	public static Date stringToDate(String string) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
			format.setLenient(false);
			return format.parse(string);
		} catch (Exception exception) {
			return null;
		}
	}

	public static List<Integer> getMonthsOfPeriod(Date dateBegin, Date dateEnd) {
		if (!isFutureDate(dateBegin))
			throw new IllegalArgumentException("Invalid date");
		if (!isPeriodValid(dateBegin, dateEnd))
			throw new IllegalArgumentException("Invalid period");

		Set<Integer> months = new LinkedHashSet<Integer>();
		Calendar begin = Calendar.getInstance();
		begin.setTime(dateBegin);
		Calendar end = Calendar.getInstance();
		end.setTime(dateEnd);
		while (begin.getTimeInMillis() < end.getTimeInMillis()) {
			months.add(begin.get(Calendar.MONTH));
			begin.add(Calendar.MONTH, 1);
		}
		months.add(end.get(Calendar.MONTH));
		return new ArrayList<Integer>(months);
	}

	public static String monthIntToString(int i) {
		Calendar calendar = Calendar.getInstance();
		calendar.set(2000, i, 1);
		return calendar.getDisplayName(Calendar.MONTH, Calendar.LONG,
				Locale.ENGLISH);
	}

	public static boolean validateAll(Date dateBegin, Date dateEnd) {

		boolean datesNotNull = (dateBegin != null && dateEnd != null);

		return datesNotNull && isFuturePeriod(dateBegin, dateEnd);
	}
	
	private static boolean isFuturePeriod(Date dateBegin, Date dateEnd) {
		boolean futureDates = isFutureDate(dateBegin) && isFutureDate(dateEnd);
		boolean validPeriod = isPeriodValid(dateBegin, dateEnd);
		return futureDates && validPeriod;
	}

	public static String daysFromTodayISO(int numberOfDays) {
		Calendar today = Calendar.getInstance();
		today.add(Calendar.DAY_OF_YEAR, numberOfDays);
		return new SimpleDateFormat("yyyy-MM-dd").format(today.getTime());
	}



}