package agileTraveler.weather;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.TagNode;

import agileTraveler.database.Database;
import agileTraveler.exceptions.CityNotFoundException;

public class WeatherHistoryParser {

	private String cityName;
	private String url;
	private List<Integer> monthList;
	private static final int MIN_TEMPERATURE = 4;
	private static final int MEAN_TEMPERATURE = 6;
	private static final int MAX_TEMPERATURE = 2;

	public WeatherHistoryParser(String cityName) throws ClassNotFoundException,
			SQLException, CityNotFoundException {
		this.cityName = cityName;
		String cityCode = getCityCode();
		this.url = "http://www.weather.com/weather/climatology/monthly/"
				+ cityCode;
	}

	public String getUrl() {
		return url;
	}

	private String getCityCode() throws ClassNotFoundException, SQLException,
			CityNotFoundException {
		String locationCode = null;
		Database db = Database.getDatabase();
		locationCode = db.getCityCode(cityName);
		return locationCode;
	}

	public void setUrl(String url) throws ClassNotFoundException, SQLException,
			CityNotFoundException {
		String cityCode = getCityCode();
		this.url = url + cityCode + ".html";
	}

	public void setMonthList(List<Integer> monthList) {
		this.monthList = monthList;
	}

	public List<WeatherHistory> getWeatherHistories()
			throws MalformedURLException, IOException {
		TagNode[] trTags = getMonthLines();

		List<WeatherHistory> weatherHistories = new ArrayList<WeatherHistory>();

		for (Integer month : monthList) {
			WeatherHistory weatherHistory = getWeatherHistoryForAMonth(month,
					trTags);
			weatherHistories.add(weatherHistory);
		}
		System.out.println("INFO: finished weather history");
		return weatherHistories;
	}

	private TagNode[] getMonthLines() throws MalformedURLException, IOException {
		HtmlCleaner cleaner = new HtmlCleaner();
		System.out.println("INFO: downloading url " + url);
		TagNode node = cleaner.clean(new URL(url));
		System.out.println("INFO: finished downloading url " + url);
		TagNode table = node.getElementsByAttValue("id", "f5", true, false)[0];
		TagNode[] trTags = table.getElementsByName("tr", true);
		return trTags;
	}

	// The table retrieved from html start counting months from 2, for instance
	// January is month(0) = 2
	private WeatherHistory getWeatherHistoryForAMonth(int month,
			TagNode[] trTags) {
		TagNode[] tdTags = trTags[2 + month].getElementsByName("td", true);
		WeatherHistory weatherHistory = new WeatherHistory();
		weatherHistory.setMonth(month);
		String meanTemperature = tdTags[MEAN_TEMPERATURE].getText().toString();
		weatherHistory.setMeanTemperature(meanTemperature);

		String maxTemperature = tdTags[MAX_TEMPERATURE].getText().toString();
		weatherHistory.setMaxTemperature(maxTemperature);

		String minTemperature = tdTags[MIN_TEMPERATURE].getText().toString();
		weatherHistory.setMinTemperature(minTemperature);

		return weatherHistory;
	}

	
	
}
