package agileTraveler.weather;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.InputSource;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.Weather;

public class WeatherParser {
	private String city;
	private Weather weather;
	private List<WeatherForecast> weatherForecasts;
	private Element xmlRoot;

	public WeatherParser(String city) throws JDOMException, IOException,
			CityNotFoundException {
		this.city = city;

		this.xmlRoot = getXmlRoot();

		checkIfCityExists();
	}

	public WeatherParser(String city, String xmlFilename) throws JDOMException,
			IOException, CityNotFoundException {
		this.city = city;

		InputStream file = getClass().getResourceAsStream("/" + xmlFilename);
		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(file);
		this.xmlRoot = doc.getRootElement().getChild("weather");

		checkIfCityExists();
	}

	public void useXML(String filename) {

	}

	private Attribute getXmlAttribute(String elementPath) {
		return getXmlAttribute(xmlRoot, elementPath);
	}

	private Attribute getXmlAttribute(Element element, String elementPath) {
		String nodes[] = elementPath.split("\\.");
		for (String node : nodes)
			element = element.getChild(node);
		return element.getAttribute("data");
	}

	private Weather parseWeather() throws DataConversionException {
		Weather weather = new Weather(city);
		weather.setCondition(getXmlAttribute("current_conditions.condition")
				.getValue());
		weather.setTemperature(getXmlAttribute("current_conditions.temp_f")
				.getIntValue());
		String humidity = getXmlAttribute("current_conditions.humidity")
				.getValue();
		humidity = humidity.substring(10, humidity.length() - 1);
		weather.setHumidity(Integer.parseInt(humidity));
		weather.setIconUrl("http://www.google.com"
				+ getXmlAttribute("current_conditions.icon").getValue());
		weather.setWindCondition(getXmlAttribute(
				"current_conditions.wind_condition").getValue());

		System.out.println("INFO: finished current weather.");

		return weather;
	}

	private void checkIfCityExists() throws CityNotFoundException {
		if (xmlRoot.getChild("forecast_information") == null) {
			throw new CityNotFoundException("City not found: " + city);
		}
	}

	private Element getXmlRoot() throws JDOMException, IOException {
		String url = "http://www.google.com/ig/api?ie=utf-8&oe=utf-8&weather=";
		InputSource xml = null;
		url += URLEncoder.encode(city, "UTF-8");
		System.out.println("INFO: downloading url " + url);
		xml = new InputSource(url);

		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(xml);
		return doc.getRootElement().getChild("weather");
	}

	public Weather getWeather() throws DataConversionException {
		if (weather == null) {
			weather = parseWeather();
		}
		return weather;
	}

	public List<WeatherForecast> getWeatherForecast() {
		if (weatherForecasts == null) {
			weatherForecasts = parseWeatherForecasts();
		}
		return weatherForecasts;
	}

	@SuppressWarnings("unchecked")
	private List<WeatherForecast> parseWeatherForecasts() {
		List<WeatherForecast> weatherForecasts = new ArrayList<WeatherForecast>();
		List<Element> forecasts = xmlRoot.getChildren("forecast_conditions");
		for (Element forecast : forecasts) {
			weatherForecasts.add(parseWeatherForecast(forecast));
		}
		System.out.println("INFO: finished weather forecast.");
		return weatherForecasts;
	}

	private WeatherForecast parseWeatherForecast(Element forecast) {
		WeatherForecast weatherForecast = new WeatherForecast();
		weatherForecast.setDayOfWeek(getXmlAttribute(forecast, "day_of_week")
				.getValue());
		weatherForecast.setLow(getXmlAttribute(forecast, "low").getValue());
		weatherForecast.setHigh(getXmlAttribute(forecast, "high").getValue());
		weatherForecast.setIconUrl("http://www.google.com"
				+ getXmlAttribute(forecast, "icon").getValue());
		weatherForecast.setCondition(getXmlAttribute(forecast, "condition")
				.getValue());
		return weatherForecast;
	}

}
