package agileTraveler.weather;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.ini4j.Ini;
import org.ini4j.Profile.Section;
import org.jdom.DataConversionException;
import org.jdom.JDOMException;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.Weather;
import agileTraveler.utilities.IniUtils;

public class WeatherSearchEngineMock implements WeatherSearchEngineFactory {

	private Weather weather;
	private List<WeatherForecast> forecasts;
	private List<WeatherHistory> weatherHistories;

	@Override
	public Weather getWeather(String city) throws JDOMException, IOException,
			CityNotFoundException {

		if (!weather.getCity().equals(city))
			throw new CityNotFoundException(city);

		return weather;
	}

	@Override
	public void initialize() throws IOException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {

		Ini mockIni = new Ini(getClass().getResourceAsStream("/mock.ini"));

		weather = new Weather("foo");
		IniUtils.fillObject(mockIni.getAll("CurrentWeather").get(0), 0,
				weather, Weather.class);

		forecasts = new ArrayList<WeatherForecast>();
		Section days = mockIni.getAll("WeatherForecast").get(0);
		String firstKey = days.keySet().iterator().next();
		int size = days.getAll(firstKey).size();

		for (int i = 0; i < size; i++) {
			WeatherForecast weatherForecast = new WeatherForecast();
			forecasts.add(weatherForecast);
			IniUtils
					.fillObject(days, i, weatherForecast, WeatherForecast.class);
		}

		weatherHistories = new ArrayList<WeatherHistory>();
		Section months = mockIni.getAll("WeatherHistory").get(0);

		firstKey = months.keySet().iterator().next();
		size = months.getAll(firstKey).size();

		for (int i = 0; i < size; i++) {
			WeatherHistory weatherHistory = new WeatherHistory();
			weatherHistories.add(weatherHistory);
			IniUtils
					.fillObject(months, i, weatherHistory, WeatherHistory.class);
		}
	}

	@Override
	public List<WeatherForecast> getWeatherForecast(String city)
			throws DataConversionException, JDOMException, IOException,
			CityNotFoundException {
		return forecasts;
	}

	@Override
	public List<WeatherHistory> getWeatherHistory(String city,
			Date departureDate, Date returnDate) {
		return weatherHistories;
	}

}
