package agileTraveler.weather;

import java.util.Date;
import java.util.List;

import agileTraveler.model.Weather;

public class WeatherSearcher {

	private WeatherSearchEngineFactory weatherSearchEngineFactory;

	public WeatherSearcher(String weatherFactoryClassName)
			throws WeatherSearchException {
		try {
			if (weatherFactoryClassName != null) {
				weatherSearchEngineFactory = (WeatherSearchEngineFactory) Class
						.forName(weatherFactoryClassName).newInstance();
			} else {
				weatherSearchEngineFactory = new WeatherSearchEngine();
			}
			weatherSearchEngineFactory.initialize();
		} catch (Exception e) {
			throw new WeatherSearchException(
					"Error initializing weather search factory.", e);
		}
	}

	public Weather getCurrentWeather(String city) throws WeatherSearchException {
		try {
			return weatherSearchEngineFactory.getWeather(city);
		} catch (Exception e) {
			throw new WeatherSearchException(e.getMessage(), e);
		}
	}

	public List<WeatherForecast> getWeatherForecast(String city)
			throws WeatherSearchException {
		try {
			return weatherSearchEngineFactory.getWeatherForecast(city);
		} catch (Exception e) {
			throw new WeatherSearchException(e.getMessage(), e);
		}
	}

	public List<WeatherHistory> getWeatherHistory(String city, Date start,
			Date end) {
		return weatherSearchEngineFactory.getWeatherHistory(city, start, end);
	}
}