package agileTraveler.airTicketPrice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import agileTraveler.exceptions.AirTicketPricesNotFoundException;
import agileTraveler.exceptions.CityNotFoundException;

public class AirCompanyTicketPricesParserTest {

	private static AirCompanyTicketPricesParser parser;
	
	@BeforeClass
	public static void setUp(){
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/airTicketPriceGRU2PAR.html";
		parser = new AirCompanyTicketPricesParser("GRU", "PAR", "07/01/10", "08/01/10");
		parser.setURL(localURL);
	}
	
	@Test
	public void shouldReturnAirTicketPrice() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertNotNull(ticketPrices);
	}
	
	@Test
	public void shouldReturnFiveAirCompanyTicketPrices() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals(5, ticketPrices.size());
	}
	
	@Test
	public void theFirstCompanyNameShouldBeMixAndMatchAirlines() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("Mix & Match Airlines", ticketPrices.get(0).getName());
	}
	
	@Test
	public void theSecondCompanyNameShoudlBeAirFrance() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("Air France", ticketPrices.get(1).getName());		
	}
	
	@Test
	public void theFirstCompanyLogoUrlShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("http://www.expedia.com/eta/fltnvbp.gif", ticketPrices.get(0).getLogo());				
	}
	
	@Test
	public void theFifthCompanyLogoUrlShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("http://www.expedia.com/pubspec/images/airlines/smKL.gif", ticketPrices.get(4).getLogo());		
	}
	
	@Test
	public void theFirstNonStopValueShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("from $2802", ticketPrices.get(0).getPriceForNonStop());
	}
	
	@Test
	public void theFirstOneStopValueShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("from $2902", ticketPrices.get(0).getPriceForOneStop());
	}
	
	@Test
	public void theFifthNonStopValueShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("---", ticketPrices.get(4).getPriceForNonStop());
	}
	
	@Test
	public void theThirdOneStopValueShouldBeRight() throws Exception {
		List<AirCompanyTicketPrices> ticketPrices = parser.getAirTicketPrices();
		assertEquals("from $3385", ticketPrices.get(2).getPriceForOneStop());
	}
	
	@Test(expected=CityNotFoundException.class)
	public void shouldThrowCityNotFoundException() throws Exception {
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/airTicketPriceCityNotFound.html";
		AirCompanyTicketPricesParser parserE = new AirCompanyTicketPricesParser("ZZZ", 
				"PAR", "07/01/10", "08/01/10");
		
		parserE.setURL(localURL);
		
		parserE.getAirTicketPrices();
		
	}
	
	@Test(expected=AirTicketPricesNotFoundException.class)
	public void shouldThrowAirTicketPricesNotFoundException() throws Exception {
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/airTicketPricesNotFound.html";
		AirCompanyTicketPricesParser parserE = new AirCompanyTicketPricesParser("UNK", 
				"PAR", "07/01/10", "08/01/10");
		parserE.setURL(localURL);
		
		parserE.getAirTicketPrices();
	}
	
	@Test
	public void shouldReturnThreeAirTicketCompany() throws Exception {
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/fewCompanies.html";
		AirCompanyTicketPricesParser parserE = new AirCompanyTicketPricesParser("HUY", 
				"PAR", "15/07/10", "20/08/10");
		parserE.setURL(localURL);
		assertEquals (3, parserE.getAirTicketPrices().size());
	}
	
	@Test
	public void shouldReturnEmptyValueForMixNMatch() throws Exception {
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/fewCompanies.html";
		AirCompanyTicketPricesParser parserE = new AirCompanyTicketPricesParser("HUY", 
				"PAR", "15/07/10", "20/08/10");
		parserE.setURL(localURL);
		AirCompanyTicketPrices mixAndMatchTicketPrices = parserE.getAirTicketPrices().get(0);
		assertEquals ("---", mixAndMatchTicketPrices.getPriceForNonStop());		
	}
	
	@Test(expected=AirTicketPricesNotFoundException.class)
	public void shouldThrowErrorWhenNoPriceForOneOrNonStop() throws Exception {
		String localURL = "file://"+System.getProperty("user.dir") + 
		"/src/main/resources/noPriceInAirticketTable.html";
		AirCompanyTicketPricesParser parserE = new AirCompanyTicketPricesParser("UNK", 
				"PAR", "07/01/10", "08/01/10");
		parserE.setURL(localURL);
		
		parserE.getAirTicketPrices();
	}
}
