package agileTraveler.gui;

import static agileTraveler.gui.SeleniumUtil.*;
import static agileTraveler.utilities.DateUtils.daysFromTodayISO;

import org.junit.Test;

import com.thoughtworks.selenium.DefaultSelenium;

public class AirTicketIntegrationTest {

	private static DefaultSelenium client = startSelenium();

	@Test
	public void shouldTellPricesAreRoundTrip() {
		client.open(buildInitialPage("ParisTravelGuides", "AirTicketSimple", true));
		setDepartureCityAndAirport("Guarulhos International - Sao Paulo (GRU)", "GRU");
		setDestinationCityAndAirport("Paris (PAR)", "PAR");
		setDates(daysFromTodayISO(3), daysFromTodayISO(20));

		forceValidInput();
		submitForm();
		assertTextPresent("Air Tickets from Guarulhos International - Sao Paulo (GRU) to Paris (PAR) - Roundtrip");
	}
	
	@Test
	public void shouldShowMessageIfNoTravelGuideFound() {
		client.open(buildInitialPage("thisSectionDoesNotExist", "thisSectionDoesNotExist", true));
		setDepartureCityAndAirport("São Paulo", "GRU");
		setDestinationCityAndAirport("Pindamonhangaba", "");
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("Couldn't search for air tickets because of missing airport in departure or destination city.");
	}
	
	@Test
	public void shouldShowMessageIfDateIsNull() {
		client.open(buildInitialPage("thisSectionDoesNotExist", "thisSectionDoesNotExist", true));
		setDepartureCityAndAirport("São Paulo", "GRU");
		setDestinationCityAndAirport("Tokyo", "TYO");
		forceValidInput();
		submitForm();
		assertTextPresent("Couldn't search for air tickets. Please, check the date fields.");		
	}
	
	@Test
	public void shouldShowMessageIfDateIsInThePast() {
		client.open(buildInitialPage("thisSectionDoesNotExist", "thisSectionDoesNotExist", true));
		setDepartureCityAndAirport("São Paulo", "GRU");
		setDestinationCityAndAirport("Tokyo", "TYO");
		setDates(daysFromTodayISO(-2), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("Couldn't search for air tickets. Please, check the date fields.");		
	}
}
