package agileTraveler.gui;

import static org.junit.Assert.assertTrue;

import java.net.BindException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.openqa.selenium.server.RemoteControlConfiguration;
import org.openqa.selenium.server.SeleniumServer;

import com.thoughtworks.selenium.DefaultSelenium;

public class SeleniumUtil {

	public static final String TIME_OUT = "30000";

	private static SeleniumServer server;
	private static DefaultSelenium client;

	public static final String SEARCH_FORM = "searchForm";
	public static final String DEPARTURE_CITY = SEARCH_FORM + ":departure";
	public static final String DEPARTURE_AIRPORT_NAME = DEPARTURE_CITY
			+ "AirportName";
	public static final String DEPARTURE_AIRPORT_CODE = DEPARTURE_CITY
			+ "AirportCode";
	public static final String DEPARTURE_DATE = SEARCH_FORM + ":departureDate";
	public static final String DESTINATION_CITY = SEARCH_FORM + ":destination";
	public static final String DESTINATION_AIRPORT_NAME = DESTINATION_CITY
			+ "AirportName";
	public static final String DESTINATION_AIRPORT_CODE = DESTINATION_CITY
			+ "AirportCode";
	public static final String RETURN_DATE = SEARCH_FORM + ":returnDate";
	public static final String SEARCH_BUTTON = SEARCH_FORM + ":search";
	public static final String LOCATION_CODE = DESTINATION_CITY
			+ "LocationCode";
	public static final String INITIAL_PAGE = "faces/home.jsp";

	// Should be run in a @Before or @BeforeClass method
	public static DefaultSelenium startSelenium() {
		if (client != null)
			return client;

		RemoteControlConfiguration conf = new RemoteControlConfiguration();
		conf.setSingleWindow(true);
		try {
			server = new SeleniumServer(conf);
			server.start();
		} catch (BindException e) {
			System.out
					.println("Couldn't bind to :4444. Maybe selenium server is already running.");
		} catch (Exception e) {
			e.printStackTrace();
		}

		client = new DefaultSelenium("localhost", 4444, "*chrome",
				"http://localhost:8080/");
		client.start();
		return client;
	}

	public static void setDates(String departureDate, String returnDate) {
		setDepartureDate(departureDate);
		setReturnDate(returnDate);
	}

	public static void setReturnDate(String returnDate) {
		client.type(RETURN_DATE, returnDate);
	}

	public static void setDepartureDate(String departureDate) {
		client.type(DEPARTURE_DATE, departureDate);
	}

	public static void setFormValue(String id, String value) {
		client.getEval("window.document.getElementById('" + id + "').value = '"
				+ value + "';");
	}

	public static void forceValidInput() {
		forceDepartureInputValidation();
		forceDestinationInputValidation();
		validateForm();
		clearErrorMessage();
	}

	private static void clearErrorMessage() {
		client
				.getEval("window.document.getElementById('searchForm:suggestionMessage').value = '';");
	}

	public static void validateForm() {
		client.getEval("window.updateButton();");
	}

	public static void forceDepartureInputValidation() {
		client.getEval("window.document.getElementById('" + DEPARTURE_CITY
				+ "').className = 'inputValid';");
	}

	public static void forceDestinationInputValidation() {
		client.getEval("window.document.getElementById('" + DESTINATION_CITY
				+ "').className = 'inputValid';");
	}

	public static void setDepartureCityAndAirport(String city,
			String airportCode) {
		client.type(DEPARTURE_CITY, city);
		setFormValue(DEPARTURE_AIRPORT_NAME, city);
		setFormValue(DEPARTURE_AIRPORT_CODE, airportCode);
	}

	public static void setDestinationCityAndAirport(String city,
			String airportCode) {
		// Paris is the default location code
		setDestinationCityAndAirport(city, airportCode, "FRXX0076");
	}

	public static void setDestinationCityAndAirport(String city,
			String airportCode, String locationCode) {
		client.type(DESTINATION_CITY, city);
		setFormValue(DESTINATION_AIRPORT_NAME, city);
		setFormValue(DESTINATION_AIRPORT_CODE, airportCode);
		setFormValue(LOCATION_CODE, locationCode);
	}

	/**
	 * Submits the form and waits for page to load. Equivalent to
	 * submitForm(true)
	 */
	public static void submitForm() {
		validateForm();
		submitForm(true);
	}

	public static boolean canSubmitForm() {
		String disabled = client.getEval("window.document.getElementById('"
				+ SEARCH_BUTTON + "').disabled");
		boolean canSubmit = false;
		if (disabled.equals("false")) {
			canSubmit = true;
		}
		return canSubmit;
	}

	public static void submitForm(boolean waitForPageToLoad) {
		client.click(SEARCH_BUTTON);
		if (waitForPageToLoad) {
			client.waitForPageToLoad(TIME_OUT);
		}
	}

	/**
	 * Helps to load mocks. Use empty strings if you want real objects.
	 * 
	 * @param travelGuideIniSection
	 *            parisTravelGuides will use resource /parisTravelGuides.xml
	 * @param weatherIniSection
	 *            Name of the ini section for weather object
	 * @param weatherForecastIniSection
	 *            Name of the ini section for weatherForecast List
	 * @param airTicketIniSection
	 *            Name of the ini section for Air Ticket Companies
	 * @return URL to be used in client.open()
	 */
	public static String buildInitialPage(String travelGuideIniSection,
			String airTicketIniSection, boolean weatherMock) {
		Map<String, String> queryStrings = new HashMap<String, String>();
		String queryString = "";

		if (!travelGuideIniSection.isEmpty())
			queryStrings.put("travelGuideIniSection", travelGuideIniSection);
		if (!airTicketIniSection.isEmpty())
			queryStrings.put("airTicketIniSection", airTicketIniSection);
		if (weatherMock)
			queryStrings.put("searchEngineFactory", "agileTraveler.weather.WeatherSearchEngineMock");

		Iterator<String> it = queryStrings.keySet().iterator();
		String key;
		if (it.hasNext()) {
			key = it.next();
			queryString = "?" + key + "=" + queryStrings.get(key);
		}
		while (it.hasNext()) {
			key = it.next();
			queryString += "&" + key + "=" + queryStrings.get(key);
		}

		return INITIAL_PAGE + queryString;
	}

	public static void assertTextPresent(String text) {
		assertTrue("Text \"" + text + "\" not present", client
				.isTextPresent(text));
	}

	@Override
	public void finalize() {
		client.stop();
		server.stop();
		try {
			super.finalize();
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
}