package agileTraveler.gui;

import static agileTraveler.gui.SeleniumUtil.DEPARTURE_CITY;
import static agileTraveler.gui.SeleniumUtil.DEPARTURE_DATE;
import static agileTraveler.gui.SeleniumUtil.DESTINATION_CITY;
import static agileTraveler.gui.SeleniumUtil.INITIAL_PAGE;
import static agileTraveler.gui.SeleniumUtil.RETURN_DATE;
import static agileTraveler.gui.SeleniumUtil.SEARCH_BUTTON;
import static agileTraveler.gui.SeleniumUtil.SEARCH_FORM;
import static agileTraveler.gui.SeleniumUtil.*;
import static agileTraveler.utilities.DateUtils.daysFromTodayISO;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import com.thoughtworks.selenium.DefaultSelenium;

public class TestIndexPage {

	public static final String SERROR_MESSAGE = SEARCH_FORM
			+ ":suggestionMessage";
	private static DefaultSelenium client = startSelenium();

	@Before
	public void startServerAndClient() {
		client.open(INITIAL_PAGE);
	}

	@Test
	public void shouldExistsDestinationCityField() {
		assertTrue(client.isElementPresent(DESTINATION_CITY));
	}

	@Test
	public void shouldExistsSearchButton() {
		assertTrue(client.isElementPresent(SEARCH_BUTTON));
	}

	@Test
	public void shouldExistsDepartureDateField() {
		assertTrue(client.isElementPresent(DEPARTURE_DATE));
	}

	@Test
	public void shouldExistsReturnDateField() {
		assertTrue(client.isElementPresent(RETURN_DATE));
	}

	private void setCities() {
		client.type(DEPARTURE_CITY, "Rome");
		client.type(DESTINATION_CITY, "Paris");
	}

	@Test
	public void shouldNotCleanSearchFieldsAfterSearching() {
		setCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertEquals("Paris", client.getValue(SEARCH_FORM + ":destination"));
		assertEquals("Rome", client.getValue(SEARCH_FORM + ":departure"));
		assertEquals(daysFromTodayISO(1), client.getValue(SEARCH_FORM + ":departureDate"));
		assertEquals(daysFromTodayISO(2), client.getValue(SEARCH_FORM + ":returnDate"));

		submitForm(false);
		String classValue = client.getEval("window.document.getElementById('"
				+ SERROR_MESSAGE + "').className;");
		assertEquals("invisible", classValue);
	}

	@Test
	public void shouldShowMessageErrorWhenNoSelectionIsMade() {
		setCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		submitForm(false);
		String classValue = client.getEval("window.document.getElementById('"
				+ SERROR_MESSAGE + "').className;");
		assertEquals("suggestion", classValue);
	}

	@Test
	public void shouldShowMessageErrorForDestinationCityWhenNoSelectionIsMade() {
		setCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceDepartureInputValidation();
		validateForm();
		submitForm(false);
		assertTextPresent("Please, select a destination city from the list");
	}

	@Test
	public void shouldShowMessageErrorForDepartureCityWhenNoSelectionIsMade() {
		setCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceDestinationInputValidation();
		submitForm(false);
		assertTextPresent("Please, select a departure city from the list");
	}
	
	@Test
	public void shouldShowErrorMessageForPastDateOfDeparture() {
		setCities();
		setDates(daysFromTodayISO(-1), daysFromTodayISO(2));
		forceDepartureInputValidation();
		forceDestinationInputValidation();
		validateForm();
		submitForm();
		assertTextPresent("Date of departure should not be in the past.");
	}

}
