package agileTraveler.gui;

import static agileTraveler.gui.SeleniumUtil.assertTextPresent;
import static agileTraveler.gui.SeleniumUtil.buildInitialPage;
import static agileTraveler.gui.SeleniumUtil.forceValidInput;
import static agileTraveler.gui.SeleniumUtil.setDates;
import static agileTraveler.gui.SeleniumUtil.setDepartureCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.setDestinationCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.startSelenium;
import static agileTraveler.gui.SeleniumUtil.submitForm;
import static agileTraveler.utilities.DateUtils.daysFromTodayISO;

import org.apache.commons.lang.StringUtils;
import org.junit.Test;

import com.thoughtworks.selenium.DefaultSelenium;

public class TravelGuideIntegrationTest {
	private static final int HUGE_CITY_NAME_SIZE = 4096;
	private static final DefaultSelenium client = startSelenium();

	@Test
	public void shouldShowCitySearched() {
		client.open(buildInitialPage("ParisTravelGuides", "AirTicketSimple",
				true));
		setDepartureCityAndAirport("Sao Paulo (CGH)", "CGH");
		setDestinationCityAndAirport("Paris", "PAR");
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("Travel Guides for Paris");
	}

	@Test
	public void shouldShowAExpectedListOfTravelGuides() {
		client.open(buildInitialPage("ParisTravelGuides", "AirTicketSimple",
				true));
		setDepartureCityAndAirport("Sao Paulo (CGH)", "CGH");
		setDestinationCityAndAirport("Paris", "PAR");
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("Rick Steves' Paris 2009");
		assertTextPresent("Frommer's Paris 2010 (Frommer's Color Complete Guides)");
	}

	@Test
	public void shouldShowTravelGuideNotFoundMessage() {
		client.open(buildInitialPage("PindamonhangabaTravelGuides",
				"AirTicketSimple", true));
		setDepartureCityAndAirport("Sao Paulo (CGH)", "CGH");
		setDestinationCityAndAirport("Pindamonhangaba", "", "BRXX0639");
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("We did not find any travel guide for Pindamonhangaba");
	}

	@Test
	public void weatherForecastForHugeCityName() {
		// Pindamonhangaba has no travel guides, too
		client.open(buildInitialPage("PindamonhangabaTravelGuides",
				"AirTicketSimple", true));
		setDepartureCityAndAirport("Sao Paulo (CGH)", "CGH");
		setDestinationCityAndAirport(StringUtils.repeat("A",
				HUGE_CITY_NAME_SIZE), "");
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		assertTextPresent("We did not find any travel guide for");
	}
}