package agileTraveler.gui;

import static agileTraveler.gui.SeleniumUtil.DEPARTURE_DATE;
import static agileTraveler.gui.SeleniumUtil.RETURN_DATE;
import static agileTraveler.gui.SeleniumUtil.assertTextPresent;
import static agileTraveler.gui.SeleniumUtil.buildInitialPage;
import static agileTraveler.gui.SeleniumUtil.forceValidInput;
import static agileTraveler.gui.SeleniumUtil.setDates;
import static agileTraveler.gui.SeleniumUtil.setDepartureCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.setDepartureDate;
import static agileTraveler.gui.SeleniumUtil.setDestinationCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.startSelenium;
import static agileTraveler.gui.SeleniumUtil.submitForm;
import static agileTraveler.utilities.DateUtils.daysFromTodayISO;
import static org.junit.Assert.assertFalse;

import org.junit.Before;
import org.junit.Test;

import com.thoughtworks.selenium.DefaultSelenium;

public class WeatherHistoryIntegrationTest {

	protected static DefaultSelenium client = startSelenium();

	@Before
	public void loadPage() {
		client.open(buildInitialPage("ParisTravelGuides", "AirTicketSimple",
				true));
	}

	@Test
	public void shouldCheckDateOfDeparture() {
		fillCities();
		setDepartureDate("2010-30-30");
		forceValidInput();
		submitForm();
		assertTextPresent("Invalid date of departure");
	}

	private void fillCities() {
		setDepartureCityAndAirport("Sao Paulo", "SAO");
		setDestinationCityAndAirport("Paris", "PAR");
	}

	@Test
	public void shouldCheckDateOfReturn() {
		fillCities();
		client.type(DEPARTURE_DATE, daysFromTodayISO(1));
		client.type(RETURN_DATE, "2010-30-30");
		forceValidInput();
		submitForm();
		assertTextPresent("Invalid date of return");
	}

	@Test
	public void javaScriptShouldNotAllowReturnBeforeDeparture() {
		fillCities();
		setDates(daysFromTodayISO(10), daysFromTodayISO(5));
		forceValidInput();
		submitForm();
		assertFalse(client
				.isTextPresent("Date of return should be after date of departure"));
	}

	@Test
	public void shouldShowMonthlyAvereageTemperature() {
		fillCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		// today's condition
		assertTextPresent("Temperature: 15 F");
		assertTextPresent("Raining");
		assertTextPresent("Wind: N at 2 mph");
		assertTextPresent("Humidity: 50%");
		// average monthly condition
		assertTextPresent("Average Temperature in Paris");
		assertTextPresent("February");
		assertTextPresent("Avg: 23°F");
		assertTextPresent("Min: 20°F");
		assertTextPresent("Max: 25°F");
	}

	@Test
	public void shouldShowMonthlyAvereageTemperatureMoreThanAMoth() {
		fillCities();
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
		forceValidInput();
		submitForm();
		// today's condition
		assertTextPresent("Temperature: 15 F");
		assertTextPresent("Raining");
		assertTextPresent("Wind: N at 2 mph");
		assertTextPresent("Humidity: 50%");
		// average monthly condition
		assertTextPresent("Average Temperature in Paris");
		assertTextPresent("February");
		assertTextPresent("Avg: 23°F");
		assertTextPresent("Min: 20°F");
		assertTextPresent("Max: 25°F");
		assertTextPresent("March");
		assertTextPresent("Avg: 25°F");
		assertTextPresent("Min: 23°F");
		assertTextPresent("Max: 27°F");
	}
}