package agileTraveler.gui;

import static agileTraveler.gui.SeleniumUtil.DESTINATION_CITY;
import static agileTraveler.gui.SeleniumUtil.assertTextPresent;
import static agileTraveler.gui.SeleniumUtil.buildInitialPage;
import static agileTraveler.gui.SeleniumUtil.canSubmitForm;
import static agileTraveler.gui.SeleniumUtil.forceValidInput;
import static agileTraveler.gui.SeleniumUtil.setDates;
import static agileTraveler.gui.SeleniumUtil.setDepartureCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.setDestinationCityAndAirport;
import static agileTraveler.gui.SeleniumUtil.startSelenium;
import static agileTraveler.gui.SeleniumUtil.submitForm;
import static agileTraveler.utilities.DateUtils.daysFromTodayISO;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import com.thoughtworks.selenium.DefaultSelenium;

public class WeatherIntegrationTest {
	private DefaultSelenium client = startSelenium();
	private static final String URL = buildInitialPage("ParisTravelGuides",
			"AirTicketSimple", true);
	private static final String URL_NO_WEATHER_MOCK = buildInitialPage(
			"ParisTravelGuides", "AirTicketSimple", false);

	private void completeDates() {
		setDates(daysFromTodayISO(1), daysFromTodayISO(2));
	}

	@Test
	public void shouldShowCityInWeather() {
		client.open(URL);
		client.type(DESTINATION_CITY, "Paris");
		completeDates();
		assertFalse(canSubmitForm());
	}

	@Test
	public void shoulShowTodaysTemperature() {
		client.open(URL);
		fillAndSubmitForm();
		assertTextPresent("Temperature: 15 F");
	}

	private void fillAndSubmitForm() {
		client.open(URL);
		setDepartureCityAndAirport("Sao Paulo", "SAO");
		setDestinationCityAndAirport("Paris", "PAR");
		completeDates();
		forceValidInput();
		submitForm();
	}

	@Test
	public void shouldShowTodaysCondition() {
		client.open(URL);
		fillAndSubmitForm();
		assertTextPresent("Raining");
	}

	@Test
	public void shouldShowTodaysHumidity() {
		fillAndSubmitForm();
		assertTextPresent("Humidity: 50%");
	}

	@Test
	public void shouldShowTodaysWind() {
		client.open(URL);
		fillAndSubmitForm();
		assertTextPresent("Wind: N at 2 mph");
	}

	@Test
	public void shouldShowImage() {
		client.open(URL);
		fillAndSubmitForm();
		assertTrue(client
				.isElementPresent("//img[contains(@src,'conditionImage')]"));
	}

	@Test
	public void shouldShowMessageErrorWhenNoCityFound() {
		// TODO Mock with CityNotFoundException
		client.open(URL_NO_WEATHER_MOCK);
		setDepartureCityAndAirport("Sao Paulo", "SAO");
		setDestinationCityAndAirport("Serra dos Orgaos Teresopo", "",
				"BRXX3909");
		completeDates();
		forceValidInput();
		submitForm();
		assertTextPresent("Weather information issue: City not found: Serra dos Orgaos Teresopo");
	}
}