package agileTraveler.travelGuide;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.util.Collection;

import org.jdom.JDOMException;
import org.junit.BeforeClass;
import org.junit.Test;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.TravelGuide;

public class TravelGuideParserTest {
	
	private static TravelGuideParser parser;
	private static Collection<TravelGuide> travelGuides;
	
	@BeforeClass
	public static void getTravelGuides() throws JDOMException, IOException, CityNotFoundException {
		parser = new TravelGuideParser();
		parser.setCityCode("Paris");
		travelGuides = parser.getTravelGuides();
	}
	
	@Test
	public void shouldReturnACollectionOfTravelGuides() {
		assertNotNull(travelGuides);
	}
	
	@Test
	public void shouldReturnAtLeastOneTravelGuideAboutParis() {
		assertTrue(travelGuides.size() > 0);
	}
	
	@Test
	public void aTravelGuideShouldHaveTitle() {
		TravelGuide travelGuide = travelGuides.iterator().next();
		assertNotNull(travelGuide.getTitle());
	}
	
	@Test
	public void aTravelGuideShouldHaveURL() {
		TravelGuide travelGuide = travelGuides.iterator().next();
		assertNotNull(travelGuide.getUrl());
	}	
	@Test
	public void shouldURLbeDiferentFromTitle() {
		TravelGuide travelGuide = travelGuides.iterator().next();
		assertFalse(travelGuide.getUrl().equals(travelGuide.getTitle()));
	}
	
	@Test(expected = CityNotFoundException.class)
	public void shouldThrowExceptionIfCityIsNotFound() throws Exception {
		String city = "This city does not exist";
		new TravelGuideParser().setCityCode(city);
	}
	
	@Test
	public void shouldReturnNumberOfPages(){
		int numberOfPages = parser.getNumberOfPages();
		assertTrue("" + numberOfPages, numberOfPages > 10);
		assertTrue("" + numberOfPages, numberOfPages < 100);
	}
	
	@Test
	public void shouldReturnNextPage() throws Exception {
		String firstPageGuide = travelGuides.iterator().next().getTitle();
		parser.nextPage();
		String secondPageGuide = parser.getTravelGuides().iterator().next().getTitle();
		assertFalse(firstPageGuide.equals(secondPageGuide));
	}
	
	@Test
	public void shouldNotOverlapNumberOfPages() throws Exception {
		parser.firstPage();
		parser.previousPage();
		assertEquals(1, parser.getPage());
		
		parser.lastPage();
		parser.nextPage();
		assertEquals(parser.getNumberOfPages(), parser.getPage());
	}
}
